var MozrefSidebar = {
    // Objects and interfaces
    RDF: null,
    RDFC: null,
    URL : null,

    // Constants
    RDF_Seq           : null,
    RDF_type          : null,
    RDFS_seeAlso      : null,
    DC_description    : null,
    DC_title          : null,
    REF_object        : null,
    REF_method        : null,
    REF_property      : null,
    MOZREF_searchRoot : null,

    // Element references
    mFullObjectTree : null,
    mObjectList     : null,
    gDescription    : null,
    gLastQuery      : null,

    init : function() {
        this.RDF = Components
            .classes['@mozilla.org/rdf/rdf-service;1']
            .getService()
            .QueryInterface(Components.interfaces.nsIRDFService);

        this.RDFC = Components
            .classes["@mozilla.org/rdf/container-utils;1"]
            .getService(Components.interfaces.nsIRDFContainerUtils);

        this.URL = new Components.Constructor("@mozilla.org/network/standard-url;1", "nsIURL");
        this.RDF_Seq           = this.RDF.GetResource( "http://www.w3.org/1999/02/22-rdf-syntax-ns#Seq" );
        this.RDF_type          = this.RDF.GetResource( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
        this.RDFS_seeAlso      = this.RDF.GetResource( "http://www.w3.org/2000/01/rdf-schema#seeAlso" );
        this.DC_description    = this.RDF.GetResource( "http://purl.org/dc/elements/1.1/description" );
        this.DC_title          = this.RDF.GetResource( "http://purl.org/dc/elements/1.1/title" );
        this.REF_object        = this.RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#object" );
        this.REF_method        = this.RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#method" );
        this.REF_property      = this.RDF.GetResource( "http://mozref.com/2004/07/14-object-reference#property" );
        this.MOZREF_searchRoot = this.RDF.GetResource("mozref:search");

        this.mFullObjectTree = document.getElementById("mozrefObjectFullTree");
        //this.mObjectList = document.getElementById("mozrefObjectList");
        this.gDescription = document.getElementById("mozrefDescription");
        this.gLabel = document.getElementById("mozrefLabel");

        this.mFullObjectTree.setAttribute('datasources', mozrefService.getUrl() + "?style=rdf");
        //this.mObjectList.setAttribute('datasources', mozrefService.getUrl() + "?style=rdf");

        this.makeSearchRoot();
        this.emptySearchResults();
        this.gLastQuery = "";
        this.searchContents("");
    },

    onOpenNode : function(event) {
        var target = event.target;
        if (target.nodeName == 'treechildren')
            target = target.parentNode;
        var uri = this.getSelectedURI(target);
        if (uri == null) return null;

        var source = this.RDF
            .GetResource(uri)
            .QueryInterface(Components.interfaces.nsIRDFNode);

        var url = this.mFullObjectTree
            .database
            .GetTarget(source, this.RDFS_seeAlso, true)
            .QueryInterface(Components.interfaces.nsIRDFResource)
            .Value;

        switch (mozrefService.getOpenAction()) {
            case "newWindow":
                openNewWindowWith(url, null, false);
                break;
            case "newTab":
                openNewTabWith(url, null, event, false);
                break;
            case "none":
                break;
            case "currWindow":
                // Fall-through
            default:
                openTopWin(url);
                break;
        }
    },

    onSelectNode : function(event) {
        var uri = this.getSelectedURI(event.target);
        this.gDescription.value = "";
        this.gLabel.value = "";
        if (uri == null)
            return null;

        var source = this.RDF
            .GetResource(uri)
            .QueryInterface(Components.interfaces.nsIRDFNode);

        var name = this.mFullObjectTree
            .database
            .GetTarget(source, this.DC_title, true)
            .QueryInterface(Components.interfaces.nsIRDFLiteral)
            .Value;

        var description = this.mFullObjectTree
            .database
            .GetTarget(source, this.DC_description, true)
            .QueryInterface(Components.interfaces.nsIRDFLiteral)
            .Value;

        // Set the RDF context for the bottom text box
        this.gLabel.value = name;
        this.gDescription.value = description;
    },

    getSelectedURI : function(target) {
        var currentIndex = target.view.selection.currentIndex;
        if (currentIndex < 0) return null;
        return target.view.getResourceAtIndex(currentIndex).Value;
    },

    searchContents : function(searchText) {
        if (searchText == "") {
            this.mFullObjectTree.setAttribute('ref', mozrefService.getUrl());
            //this.mObjectList.setAttribute('ref', mozrefService.getUrl());
            this.gLastQuery = "";
            return;
        }
        this.mFullObjectTree.setAttribute('ref', this.MOZREF_searchRoot.Value);
        //this.mObjectList.setAttribute('ref', this.MOZREF_searchRoot.Value);

        // Determine how we are going to change an existing search criteria
        if (this.gLastQuery == "") {
            // This is a new search, just do it.
            this.emptySearchResults();
            this.makeSearchResults(searchText);
        } else {
            // We're modifying a search; check to see how it should be done
            if (this.gLastQuery.length < searchText.length && searchText.indexOf(this.gLastQuery) == 0) {
                // Handle when an extra character is added to an existing search.
                this.refineSearchResults(searchText);
            } else if (this.gLastQuery.length > searchText.length && this.gLastQuery.indexOf(searchText) == 0) {
                // Handle when a character has been removed
                this.makeSearchResults(searchText);
            } else {
                // Bah, its a new search...run it.
                this.emptySearchResults();
                this.makeSearchResults(searchText);
            }
        }
        this.gLastQuery = searchText;
        this.mFullObjectTree.builder.rebuild();
    },

    refineSearchResults : function(query) {
        query = query.toLowerCase();
        var ds = this.mFullObjectTree.database;
        var container = this.getSearchRoot();

        var removeArray = new Array();
        var children = container.GetElements();
        while (children.hasMoreElements()) {
            var child = children.getNext();
            if (child instanceof Components.interfaces.nsIRDFResource) {
                child = child.QueryInterface(Components.interfaces.nsIRDFResource);

                if (child.Value.toLowerCase().indexOf(query) == -1) {
                    removeArray.push(child);
                }
            }
        }

        for (var idx = 0; idx < removeArray.length; idx++) {
            container.RemoveElement(removeArray[idx], true);
        }
    },

    makeSearchResults : function(query) {
        query = query.toLowerCase();
        var ds = this.mFullObjectTree.database;
        var predicates = new Array(this.REF_object, this.REF_method, this.REF_property);
        var container = this.getSearchRoot();

        for (var idx in predicates) {
            var targets = ds.GetSources(this.RDF_type, predicates[idx], true);
            while (targets.hasMoreElements()) {
                var uri = targets
                    .getNext()
                    .QueryInterface(Components.interfaces.nsIRDFResource);

                var title = ds.GetTarget(uri, this.DC_title, true);
                if (title instanceof Components.interfaces.nsIRDFLiteral) {
                    title = title.QueryInterface(Components.interfaces.nsIRDFLiteral);
                } else {
                    continue;
                }

                if (title.Value.toLowerCase().indexOf(query) > -1) {
                    // Skip if we should not add this again
                    if (this.RDFC.indexOf(ds, this.MOZREF_searchRoot, uri) > -1) {
                        continue;
                    }

                    try {
                        container.AppendElement(uri);
                    } catch (e) {
                        dump("Aack, we had an exception " + e + "\n");
                    }
                }
            }
        }
    },

    emptySearchResults : function() {
        var container = this.getSearchRoot();

        // Loop through the children, and remove them
        var results = container.GetElements();
        var idx = 0;
        while (results.hasMoreElements()) {
            results.getNext();
            idx++;
        }
        if (idx == 0)
            return;

        while (idx > 0) {
            //var renumberList = idx == 1 ? true : false;
            try {
                container.RemoveElementAt(idx, true);
            } catch (e) {
                dump("Deleting " + idx + " failed: " + e + "\n");
            }
            idx--;
        }
        this.emptySearchResults(query);
    },

    getSearchRoot : function() {
        var ds = this.mFullObjectTree.database;

        try {
            var container = Components.classes["@mozilla.org/rdf/container;1"].
                              createInstance(Components.interfaces.nsIRDFContainer);
            container.Init(ds, this.MOZREF_searchRoot);
            return container;
        } catch (e) {
            // Don't do anything, since this resource doesn't exist
            // This shouldn't happen, since we create the container
            // above, but you never know.
            return null;
        }
    },

    makeSearchRoot : function() {
        var ds = this.mFullObjectTree.database;

        var container = this.RDFC.MakeSeq(ds, this.MOZREF_searchRoot);
    }
};
